<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://competition.demo.getfoundni.co.uk/
 * @since      1.0.0
 *
 * @package    Gfni_Spot_The_Ball_Addon
 * @subpackage Gfni_Spot_The_Ball_Addon/public
 */
 

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Gfni_Spot_The_Ball_Addon
 * @subpackage Gfni_Spot_The_Ball_Addon/public
 * @author     Get Found NI <ryan@getfoundni.co.uk>
 */
class Gfni_Spot_The_Ball_Addon_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() { 

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/gfni-spot-the-ball-addon-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() { 
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/gfni-spot-the-ball-addon-public.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'alpine', 'https://cdnjs.cloudflare.com/ajax/libs/alpinejs/2.3.0/alpine.js', array(), '2.3.0', true );
	}

	public function add_spot_the_ball_photo()
	{
		if( function_exists('get_field') && get_field('enable_spot_the_ball') ):
			global $product;

			ob_start(); ?> 
			<div class="spot-the-ball" x-data="guesses()" x-init="init()">

				<!-- Trigger/Open The Modal -->
				<h4>Step 1: Register Your Guesses</h4>
				<span class="single_add_to_cart_button button alt" @click="modal = true">Open Game</span>

 
				<!-- The Modal -->
				<div id="myModal" class="modal" x-show="modal" x-cloak>

					<!-- Modal content -->
					<div class="modal-content" @click.away="modal = false" style="width: <?php echo get_field('spot_the_ball_image')["width"]; ?>px;">
						<img id="hotspot_image" src="<?php echo get_field('spot_the_ball_image')["url"]; ?>" @click="addGuess($event)" 
								width="<?php echo get_field('spot_the_ball_image')["width"]; ?>px" 
								height="<?php echo get_field('spot_the_ball_image')["height"]; ?>px"> 
						<template x-for="(guess,index) in guesses">
							<img class="cross" src="<?php echo plugin_dir_url( __FILE__ ).'images/cross.png';?>" :style="`left: ${(guess.x - 5)}px;top: ${(guess.y - 5)}px;`">
						</template>
						
						<div id="spotlist" class="spotlist"> 
							<template x-for="(guess,index) in guesses">
								<div class="cords" @click="removeGuess(index)" x-text="formatGuess(guess)"></div>
							</template>
							<p x-show="!hasGuesses()">Click on the image above to make your first guess!</p> 
							<a class="btn" @click.prevent="modal = false">Save</a>
						</div>
					</div>

				</div>

				<div id="spotlist" class="spotlist" x-cloak x-show="hasGuesses()"> 
					<h4>Your Guess Coordinates!</h4> 
					<template x-for="(guess,index) in guesses">
						<div class="cords" @click="removeGuess(index)" x-text="formatGuess(guess)"></div>
					</template>
				</div> 
				<template x-for="(guess,index) in guesses">
					<input name="guesses[]" type="hidden" @click="removeGuess(index)" x-text="formatGuess(guess)" :value="JSON.stringify(guess)"> 
				</template>

				<h4>Step 2: Add To Basket</h4>

				 
				<script>
					function guesses() {
						return {
							guesses: [],
							modal: false,
							price: <?php echo $product->get_price() * 100; ?>,
							init(){
								var x = document.getElementsByName("add-to-cart")[0];
								if (this.hasGuesses()) {
									x.disabled = false;
								} else {
									x.disabled = true;
								}
							},
							hasGuesses(){
								return this.guesses.length
							},
							addGuess(event) {
								const getMousePos = (evt) => {
									const pos = evt.currentTarget.getBoundingClientRect();
									return {
										x: Math.round(evt.clientX - pos.left),
										y: Math.round(evt.clientY - pos.top)
									};
								};							
								this.guesses.push(getMousePos(event)); 

								this.changeQty(); 
							},
							formatGuess(g){
								return 'x:' + g.x +', y:' + g.y;
							},
							removeGuess(id) { 
								this.guesses.splice(id,1);
								this.changeQty(); 
							},
							changeQty(){ 
								const qty = document.getElementsByClassName('qty')[0];
								const button = document.getElementsByName("add-to-cart")[0];
								qty.value = this.guesses.length;   
								if(this.guesses.length > 0){ 
									button.innerHTML = "Enter Now for <?php echo get_woocommerce_currency_symbol(); ?>" + ((this.price * this.guesses.length) / 100).toFixed(2);
								}
								this.init();
							}
						}
					}
				</script>

				<style lang="css">
					.woocommerce .quantity{display: none;}
				</style>
			</div>
			<?php echo ob_get_clean();
		endif;
	} 


	function validate_custom_field( $passed, $product_id, $quantity ) { 
		if( function_exists('get_field') && get_field('enable_spot_the_ball',$product_id) ):
			if( ! isset( $_POST['guesses'] ) ) { 
				$passed = false;
				WC()->cart->empty_cart();
				wc_add_notice('Please make a guess before adding to basket.', 'error' );
			}
		endif;
		return $passed;
	}
  
	public function wc_cart_item_quantity( $product_quantity, $cart_item_key, $cart_item ){
		if( is_cart() ){
			if( function_exists('get_field') && get_field('enable_spot_the_ball',$cart_item['product_id']) ): 
				$product_quantity = sprintf( '%2$s <input type="hidden" name="cart[%1$s][qty]" value="%2$s" />', $cart_item_key, $cart_item['quantity'] );
			endif;
		}
		return $product_quantity;
	}

	function guesses_cart_item_name( $name, $cart_item, $cart_item_key ) {
		//WC()->cart->empty_cart();
		if( isset( $cart_item['guess'] ) ) {
			$name .= '<p class="variation" style="margin-bottom:0px;">Guess: x'. json_decode( stripslashes($cart_item['guess']),true)['x'] . ' y' . json_decode(stripslashes($cart_item['guess']),true)['y'].'</p>';
		}
		return $name;
	}
 
	// Split items by their quantity units 
	public function split_cart_items_by_quantity( $cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data ) {
		if( function_exists('get_field') && get_field('enable_spot_the_ball',$product_id) ): 
		
			if( ! empty( $_POST['guesses'] ) ) {     
				if ( $quantity == 1 ) { 
					$cart_item_data['guess'] = $_POST['guesses'][0];
				}

				if ( $quantity > 1 ) {  
					WC()->cart->remove_cart_item( $cart_item_key);
					$guess = 0;
					for ( $i = 1; $i <= $quantity; $i++ ) {
						// Make each quantity item unique and separated
						$cart_item_data['unique_key'] = md5( microtime() . rand());
						$cart_item_data['guess'] = $_POST['guesses'][$guess];

						$guess++; 

						WC()->cart->add_to_cart( $product_id, 1, $variation_id, $variation, $cart_item_data );
					}
				}
			}

		endif;
	}

	// Make added cart item as a unique and separated (works with ajax add to cart too)
	public function filter_add_cart_item_data( $cart_item_data, $product_id, $variation_id, $quantity ) { 

		if( ! empty( $_POST['guesses'] ) ) {   
			if ( ! isset($cart_item_data['unique_key']) ) { 
				$cart_item_data['unique_key'] = md5( microtime().rand() . "Hi Dad!"  );
				$cart_item_data['guess'] = $_POST['guesses'][0];
			} 
		}
		return $cart_item_data;

	} 

	public function cfwc_add_custom_data_to_order( $item, $cart_item_key, $values, $order ) {
		foreach( $item as $cart_item_key => $values ) {
			if( isset( $values['guess'] ) ) {
				$item->add_meta_data('Guess', 'x '.json_decode( stripslashes($values['guess']),true)['x'] . ' y ' . json_decode(stripslashes($values['guess']),true)['y'], true );
			}
		}
	}
}
